<?php

namespace Drupal\media_pdf_thumbnail\Manager;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\File\FileSystem;
use Drupal\Core\State\State;
use Drupal\file\FileInterface;
use Drupal\media\MediaInterface;

/**
 * Class MediaPdfThumbnailImageManager.
 *
 * @package Drupal\media_pdf_thumbnail\Manager
 */
class MediaPdfThumbnailImageManager {

  const VALID_MIME_TYPE = ['application/pdf'];

  /**
   * MediaPdfThumbnailManager.
   *
   * @var \Drupal\media_pdf_thumbnail\Manager\MediaPdfThumbnailImagickManager
   */
  protected $mediaPdfThumbnailImagickManager;

  /**
   * EntityTypeManager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * FileSystem.
   *
   * @var \Drupal\Core\File\FileSystem
   */
  protected $fileSystem;

  /**
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * @var \Drupal\Core\State\State
   */
  protected $state;

  /**
   * MediaPdfThumbnailManager constructor.
   *
   * @param \Drupal\media_pdf_thumbnail\Manager\MediaPdfThumbnailImagickManager $mediaPdfThumbnailImagickManager
   *   $mediaPdfThumbnailImagickManager.
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   *   EntityTypeManager.
   * @param \Drupal\Core\File\FileSystem $fileSystem
   *   FileSystem.
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   * @param \Drupal\Core\State\State $state
   */
  public function __construct(MediaPdfThumbnailImagickManager $mediaPdfThumbnailImagickManager, EntityTypeManager $entityTypeManager, FileSystem $fileSystem, ConfigFactory $configFactory, State $state) {
    $this->mediaPdfThumbnailImagickManager = $mediaPdfThumbnailImagickManager;
    $this->entityTypeManager = $entityTypeManager;
    $this->fileSystem = $fileSystem;
    $this->configFactory = $configFactory;
    $this->state = $state;
  }

  /**
   * @param \Drupal\media\MediaInterface $media
   * @param $fieldName
   * @param $thumbnailType
   *
   * @return false
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function createThumbnail(MediaInterface $media, $fieldName, $thumbnailType) {

    if (empty($media)) {
      return FALSE;
    }

    if ($thumbnailType == 'pdf') {
      $entityTypeId = $media->getEntityTypeId();
      if ($entityTypeId == 'media' && $fieldName) {
        if ($media->hasField($fieldName) && !empty($media->get($fieldName)->getValue())) {
          $fileEntity = $this->getFileEntity($media->get($fieldName)->getValue()[0]['target_id']);
          if ($fileEntity && in_array($fileEntity->getMimeType(), self::VALID_MIME_TYPE)) {
            $fileEntityInfo = !$this->generatePdfImage($fileEntity) ?: $this->createThumbnailFileEntity($fileEntity->getFileUri());
            if (!empty($fileEntityInfo)) {
              $this->setImageToMediaThumbnail($media, $fileEntityInfo);
              $media->save();
            }
          }
        }
      }
    }
    else {
      $this->restoreGenericThumbnail($media);
    }
  }

  /**
   * @param $fid
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getFileEntity($fid) {
    return $this->entityTypeManager->getStorage('file')->load($fid);
  }

  /**
   * @param \Drupal\file\FileInterface $fileEntity
   *
   * @return bool|string|null
   */
  protected function generatePdfImage(FileInterface $fileEntity) {
    $fileInfos = $this->getFileInfos($fileEntity);
    if (!empty($fileInfos['source']) && !empty($fileInfos['destination'])) {
      return $this->mediaPdfThumbnailImagickManager->generateImageFromPDF($fileInfos['source'], $fileInfos['destination']);
    }
    return NULL;
  }

  /**
   * @param \Drupal\file\FileInterface $fileEntity
   *
   * @return array
   */
  protected function getFileInfos(FileInterface $fileEntity) {
    $fileUri = $fileEntity->getFileUri();
    $sourcePath = $this->fileSystem->realpath($fileUri);
    $destinationPath = $sourcePath . '.jpeg';
    return ['source' => $sourcePath, 'destination' => $destinationPath];
  }

  /**
   * Create file entity.
   *
   * @param string $fileUri
   *   File uri.
   *
   * @return array File entity id.
   *   File entity id.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function createThumbnailFileEntity($fileUri) {
    $fileUriArray = explode('/', $fileUri);
    $filenameNoExtension = str_replace('.pdf', '', end($fileUriArray));
    $fileUri = str_replace('.pdf', '.pdf.jpeg', $fileUri);
    $fileEntity = $this->entityTypeManager->getStorage('file')->create([
      'uri' => $fileUri,
      'status' => FILE_STATUS_PERMANENT,
    ]);
    $fileEntity->save();
    return $fileEntity ? [
      'fid' => $fileEntity->id(),
      'filename' => $filenameNoExtension,
    ] : NULL;
  }

  /**
   * @param \Drupal\media\MediaInterface $media
   * @param $fileEntityInfo
   */
  protected function setImageToMediaThumbnail(MediaInterface $media, $fileEntityInfo) {
    $media->set('thumbnail',
      [
        'target_id' => $fileEntityInfo['fid'],
        'alt' => $fileEntityInfo['filename'],
      ]);
  }

  /**
   * @param $media
   */
  protected function restoreGenericThumbnail($media) {
    $media->set('thumbnail', [])->save();
  }
}
